@@ -1,3 +1,5 @@
+"use strict";
+
 var assert = require("assert");
 var sourceMap = require("source-map");
 var printComments = require("./comments").printComments;
@@ -298,6 +300,7 @@ function genericPrintNoParens(path, options, print) {
     case "Identifier":
         return concat([
             fromString(n.name, options),
+            n.optional ? "?" : "",
             path.call(print, "typeAnnotation")
         ]);
 
@@ -306,13 +309,24 @@ function genericPrintNoParens(path, options, print) {
     case "RestProperty": // Babel 6 for ObjectPattern
     case "SpreadProperty":
     case "SpreadPropertyPattern":
+    case "ObjectTypeSpreadProperty":
     case "RestElement":
-        return concat(["...", path.call(print, "argument")]);
+        return concat([
+            "...",
+            path.call(print, "argument"),
+            path.call(print, "typeAnnotation")
+        ]);
 
     case "FunctionDeclaration":
     case "FunctionExpression":
-        if (n.async)
+    case "TSDeclareFunction":
+        if (n.declare) {
+            parts.push("declare ");
+        }
+
+        if (n.async) {
             parts.push("async ");
+        }
 
         parts.push("function");
 
@@ -331,29 +345,30 @@ function genericPrintNoParens(path, options, print) {
             "(",
             printFunctionParams(path, options, print),
             ")",
-            path.call(print, "returnType"),
-            " ",
-            path.call(print, "body")
+            path.call(print, "returnType")
         );
 
+        if (n.body) {
+            parts.push(" ", path.call(print, "body"));
+        }
+
         return concat(parts);
 
     case "ArrowFunctionExpression":
-        if (n.async)
+        if (n.async) {
             parts.push("async ");
+        }
 
         if (n.typeParameters) {
             parts.push(path.call(print, "typeParameters"));
         }
 
-        if (
-            !options.arrowParensAlways &&
+        if (! options.arrowParensAlways &&
             n.params.length === 1 &&
-            !n.rest &&
+            ! n.rest &&
             n.params[0].type === 'Identifier' &&
-            !n.params[0].typeAnnotation &&
-            !n.returnType
-        ) {
+            ! n.params[0].typeAnnotation &&
+            ! n.returnType) {
             parts.push(path.call(print, "params", 0));
         } else {
             parts.push(
@@ -369,13 +384,7 @@ function genericPrintNoParens(path, options, print) {
         return concat(parts);
 
     case "MethodDefinition":
-        if (n.static) {
-            parts.push("static ");
-        }
-
-        parts.push(printMethod(path, options, print));
-
-        return concat(parts);
+        return printMethod(path, options, print);
 
     case "YieldExpression":
         parts.push("yield");
@@ -412,6 +421,9 @@ function genericPrintNoParens(path, options, print) {
         return fromString(" ").join(parts);
 
     case "ImportSpecifier":
+        if (n.importKind && n.importKind !== "value") {
+            parts.push(n.importKind + " ");
+        }
         if (n.imported) {
             parts.push(path.call(print, "imported"));
             if (n.local &&
@@ -461,6 +473,9 @@ function genericPrintNoParens(path, options, print) {
         }
         return path.call(print, "id");
 
+    case "TSExportAssignment":
+        return concat(["export = ", path.call(print, "expression")]);
+
     case "ExportDeclaration":
     case "ExportDefaultDeclaration":
     case "ExportNamedDeclaration":
@@ -480,6 +495,10 @@ function genericPrintNoParens(path, options, print) {
 
         return concat(parts);
 
+    case "TSNamespaceExportDeclaration":
+        parts.push("export as namespace ", path.call(print, "id"));
+        return maybeAddSemicolon(concat(parts));
+
     case "ExportNamespaceSpecifier":
         return concat(["* as ", path.call(print, "exported")]);
 
@@ -489,7 +508,7 @@ function genericPrintNoParens(path, options, print) {
     case "Import":
         return fromString("import", options);
 
-    case "ImportDeclaration":
+    case "ImportDeclaration": {
         parts.push("import ");
 
         if (n.importKind && n.importKind !== "value") {
@@ -499,36 +518,48 @@ function genericPrintNoParens(path, options, print) {
         if (n.specifiers &&
             n.specifiers.length > 0) {
 
-            var foundImportSpecifier = false;
+            const unbracedSpecifiers = [];
+            const bracedSpecifiers = [];
 
-            path.each(function(specifierPath) {
-                var i = specifierPath.getName();
+            path.each(function (specifierPath) {
+                const spec = specifierPath.getValue();
+                if (spec.type === "ImportSpecifier") {
+                    bracedSpecifiers.push(print(specifierPath));
+                } else if (spec.type === "ImportDefaultSpecifier" ||
+                           spec.type === "ImportNamespaceSpecifier") {
+                    unbracedSpecifiers.push(print(specifierPath));
+                }
+            }, "specifiers");
+
+            unbracedSpecifiers.forEach((lines, i) => {
                 if (i > 0) {
                     parts.push(", ");
                 }
+                parts.push(lines);
+            });
 
-                var value = specifierPath.getValue();
-
-                if (namedTypes.ImportDefaultSpecifier.check(value) ||
-                    namedTypes.ImportNamespaceSpecifier.check(value)) {
-                    assert.strictEqual(foundImportSpecifier, false);
-                } else {
-                    namedTypes.ImportSpecifier.assert(value);
-                    if (!foundImportSpecifier) {
-                        foundImportSpecifier = true;
-                        parts.push(
-                          options.objectCurlySpacing ? "{ " : "{"
-                        );
-                    }
+            if (bracedSpecifiers.length > 0) {
+                let lines = fromString(", ").join(bracedSpecifiers);
+                if (lines.getLineLength(1) > options.wrapColumn) {
+                    lines = concat([
+                        fromString(",\n").join(
+                            bracedSpecifiers
+                        ).indent(options.tabWidth),
+                        ","
+                    ]);
                 }
 
-                parts.push(print(specifierPath));
-            }, "specifiers");
+                if (unbracedSpecifiers.length > 0) {
+                    parts.push(", ");
+                }
 
-            if (foundImportSpecifier) {
-                parts.push(
-                  options.objectCurlySpacing ? " }" : "}"
-                );
+                if (lines.length > 1) {
+                    parts.push("{\n", lines, "\n}");
+                } else if (options.objectCurlySpacing) {
+                    parts.push("{ ", lines, " }");
+                } else {
+                    parts.push("{", lines, "}");
+                }
             }
 
             parts.push(" from ");
@@ -537,6 +568,7 @@ function genericPrintNoParens(path, options, print) {
         parts.push(path.call(print, "source"), ";");
 
         return concat(parts);
+    }
 
     case "BlockStatement":
         var naked = path.call(function(bodyPath) {
@@ -688,14 +720,9 @@ function genericPrintNoParens(path, options, print) {
         return concat(parts);
 
     case "ClassMethod": // Babel 6
-        if (n.static) {
-            parts.push("static ");
-        }
-
-        return concat([parts, printObjectMethod(path, options, print)]);
-
     case "ObjectMethod": // Babel 6
-        return printObjectMethod(path, options, print);
+    case "TSDeclareMethod":
+        return printMethod(path, options, print);
 
     case "Decorator":
         return concat(["@", path.call(print, "expression")]);
@@ -767,12 +794,34 @@ function genericPrintNoParens(path, options, print) {
     case "RegExpLiteral": // Babel 6 Literal split
         return fromString(n.extra.raw);
 
+    case "BigIntLiteral": // Babel 7 Literal split
+        return fromString(n.value + "n");
+
+    case "NumericLiteral": // Babel 6 Literal Split
+        // Keep original representation for numeric values not in base 10.
+        if (n.extra &&
+            typeof n.extra.raw === "string" &&
+            Number(n.extra.raw) === n.value) {
+            return fromString(n.extra.raw, options);
+        }
+
+        return fromString(n.value, options);
+
     case "BooleanLiteral": // Babel 6 Literal split
-    case "NumericLiteral": // Babel 6 Literal split
+
     case "StringLiteral": // Babel 6 Literal split
     case "Literal":
-        if (typeof n.value !== "string")
+        // Numeric values may be in bases other than 10. Use their raw
+        // representation if equivalent.
+        if (typeof n.value === "number" &&
+            typeof n.raw === "string" &&
+            Number(n.raw) === n.value) {
+            return fromString(n.raw, options);
+        }
+
+        if (typeof n.value !== "string") {
             return fromString(n.value, options);
+        }
 
         return fromString(nodeStr(n.value, options), options);
 
@@ -828,7 +877,12 @@ function genericPrintNoParens(path, options, print) {
         return concat(parts);
 
     case "VariableDeclaration":
+        if (n.declare) {
+            parts.push("declare ");
+        }
+
         parts.push(n.kind, " ");
+
         var maxLen = 0;
         var printed = path.map(function(childPath) {
             var lines = print(childPath);
@@ -929,24 +983,23 @@ function genericPrintNoParens(path, options, print) {
         ]);
 
     case "ForOfStatement":
-        return concat([
-            "for (",
-            path.call(print, "left"),
-            " of ",
-            path.call(print, "right"),
-            ")",
-            adjustClause(path.call(print, "body"), options)
-        ]);
-
     case "ForAwaitStatement":
-        return concat([
-            "for await (",
+        parts.push("for ");
+
+        if (n.await || n.type === "ForAwaitStatement") {
+            parts.push("await ");
+        }
+
+        parts.push(
+            "(",
             path.call(print, "left"),
             " of ",
             path.call(print, "right"),
             ")",
             adjustClause(path.call(print, "body"), options)
-        ]);
+        );
+
+        return concat(parts);
 
     case "DoWhileStatement":
         var doBody = concat([
@@ -1016,13 +1069,22 @@ function genericPrintNoParens(path, options, print) {
         return concat(parts);
 
     case "CatchClause":
-        parts.push("catch (", path.call(print, "param"));
+        parts.push("catch ");
 
-        if (n.guard)
+        if (n.param) {
+            parts.push("(", path.call(print, "param"));
+        }
+
+        if (n.guard) {
             // Note: esprima does not recognize conditional catch clauses.
             parts.push(" if ", path.call(print, "guard"));
+        }
 
-        parts.push(") ", path.call(print, "body"));
+        if (n.param) {
+            parts.push(") ");
+        }
+
+        parts.push(path.call(print, "body"));
 
         return concat(parts);
 
@@ -1090,10 +1152,17 @@ function genericPrintNoParens(path, options, print) {
         return concat(["{", path.call(print, "expression"), "}"]);
 
     case "JSXElement":
-        var openingLines = path.call(print, "openingElement");
+    case "JSXFragment":
+        var openingPropName = "opening" + (
+            n.type === "JSXElement" ? "Element" : "Fragment");
 
-        if (n.openingElement.selfClosing) {
-            assert.ok(!n.closingElement);
+        var closingPropName = "closing" + (
+            n.type === "JSXElement" ? "Element" : "Fragment");
+
+        var openingLines = path.call(print, openingPropName);
+
+        if (n[openingPropName].selfClosing) {
+            assert.ok(!n[closingPropName]);
             return openingLines;
         }
 
@@ -1114,7 +1183,7 @@ function genericPrintNoParens(path, options, print) {
             }, "children")
         ).indentTail(options.tabWidth);
 
-        var closingLines = path.call(print, "closingElement");
+        var closingLines = path.call(print, closingPropName);
 
         return concat([
             openingLines,
@@ -1155,6 +1224,12 @@ function genericPrintNoParens(path, options, print) {
     case "JSXClosingElement":
         return concat(["</", path.call(print, "name"), ">"]);
 
+    case "JSXOpeningFragment":
+        return fromString("<>");
+
+    case "JSXClosingFragment":
+        return fromString("</>")
+
     case "JSXText":
         return fromString(n.value, options);
 
@@ -1188,40 +1263,72 @@ function genericPrintNoParens(path, options, print) {
         return concat(parts);
 
     case "ClassProperty":
-        if (n.static)
+        if (typeof n.accessibility === "string") {
+            parts.push(n.accessibility, " ");
+        }
+
+        if (n.static) {
             parts.push("static ");
+        }
+
+        if (n.abstract) {
+            parts.push("abstract ");
+        }
+
+        if (n.readonly) {
+            parts.push("readonly ");
+        }
 
         var key = path.call(print, "key");
+
         if (n.computed) {
             key = concat(["[", key, "]"]);
-        } else if (n.variance === "plus") {
-            key = concat(["+", key]);
-        } else if (n.variance === "minus") {
-            key = concat(["-", key]);
         }
+
+        if (n.variance) {
+            key = concat([printVariance(path, print), key]);
+        }
+
         parts.push(key);
 
-        if (n.typeAnnotation)
-            parts.push(path.call(print, "typeAnnotation"));
+        if (n.optional) {
+            parts.push("?");
+        }
 
-        if (n.value)
+        if (n.typeAnnotation) {
+            parts.push(path.call(print, "typeAnnotation"));
+        }
+
+        if (n.value) {
             parts.push(" = ", path.call(print, "value"));
+        }
 
         parts.push(";");
         return concat(parts);
 
     case "ClassDeclaration":
     case "ClassExpression":
+        if (n.declare) {
+            parts.push("declare ");
+        }
+
+        if (n.abstract) {
+            parts.push("abstract ");
+        }
+
         parts.push("class");
 
         if (n.id) {
             parts.push(
                 " ",
-                path.call(print, "id"),
-                path.call(print, "typeParameters")
+                path.call(print, "id")
             );
         }
 
+        if (n.typeParameters) {
+            parts.push(path.call(print, "typeParameters"));
+        }
+
         if (n.superClass) {
             parts.push(
                 " extends ",
@@ -1283,6 +1390,8 @@ function genericPrintNoParens(path, options, print) {
     case "MemberTypeAnnotation": // Flow
     case "TupleTypeAnnotation": // Flow
     case "Type": // Flow
+    case "TSHasOptionalTypeParameters":
+    case "TSHasOptionalTypeAnnotation":
         throw new Error("unprintable type: " + JSON.stringify(n.type));
 
     case "CommentBlock": // Babel block comment.
@@ -1426,23 +1535,32 @@ function genericPrintNoParens(path, options, print) {
 
     case "DeclareInterface":
         parts.push("declare ");
+        // Fall through to InterfaceDeclaration...
 
     case "InterfaceDeclaration":
+    case "TSInterfaceDeclaration":
+        if (n.declare) {
+            parts.push("declare ");
+        }
+
         parts.push(
-            fromString("interface ", options),
+            "interface ",
             path.call(print, "id"),
             path.call(print, "typeParameters"),
             " "
         );
 
-        if (n["extends"]) {
+        if (n["extends"] && n["extends"].length > 0) {
             parts.push(
                 "extends ",
-                fromString(", ").join(path.map(print, "extends"))
+                fromString(", ").join(path.map(print, "extends")),
+                " "
             );
         }
 
-        parts.push(" ", path.call(print, "body"));
+        if (n.body) {
+            parts.push(path.call(print, "body"));
+        }
 
         return concat(parts);
 
@@ -1475,12 +1593,8 @@ function genericPrintNoParens(path, options, print) {
         return path.call(print, "value");
 
     case "ObjectTypeIndexer":
-        var variance =
-            n.variance === "plus" ? "+" :
-            n.variance === "minus" ? "-" : "";
-
         return concat([
-            variance,
+            printVariance(path, print),
             "[",
             path.call(print, "id"),
             ": ",
@@ -1490,12 +1604,8 @@ function genericPrintNoParens(path, options, print) {
         ]);
 
     case "ObjectTypeProperty":
-        var variance =
-            n.variance === "plus" ? "+" :
-            n.variance === "minus" ? "-" : "";
-
         return concat([
-            variance,
+            printVariance(path, print),
             path.call(print, "key"),
             n.optional ? "?" : "",
             ": ",
@@ -1522,6 +1632,7 @@ function genericPrintNoParens(path, options, print) {
 
     case "DeclareTypeAlias":
         parts.push("declare ");
+        // Fall through to TypeAlias...
 
     case "TypeAlias":
         return concat([
@@ -1533,6 +1644,29 @@ function genericPrintNoParens(path, options, print) {
             ";"
         ]);
 
+    case "DeclareOpaqueType":
+        parts.push("declare ");
+        // Fall through to OpaqueType...
+
+    case "OpaqueType":
+        parts.push(
+            "opaque type ",
+            path.call(print, "id"),
+            path.call(print, "typeParameters")
+        );
+
+        if (n["supertype"]) {
+            parts.push(": ", path.call(print, "supertype"));
+        }
+
+        if (n["impltype"]) {
+            parts.push(" = ", path.call(print, "impltype"));
+        }
+
+        parts.push(";");
+
+        return concat(parts);
+
     case "TypeCastExpression":
         return concat([
             "(",
@@ -1548,15 +1682,21 @@ function genericPrintNoParens(path, options, print) {
             fromString(", ").join(path.map(print, "params")),
             ">"
         ]);
+
+    case "Variance":
+        if (n.kind === "plus") {
+            return fromString("+");
+        }
+
+        if (n.kind === "minus") {
+            return fromString("-");
+        }
+
+        return fromString("");
+
     case "TypeParameter":
-        switch (n.variance) {
-            case 'plus':
-                parts.push('+');
-                break;
-            case 'minus':
-                parts.push('-');
-                break;
-            default:
+        if (n.variance) {
+            parts.push(printVariance(path, print));
         }
 
         parts.push(path.call(print, 'name'));
@@ -1586,6 +1726,519 @@ function genericPrintNoParens(path, options, print) {
     case "NullTypeAnnotation":
         return fromString("null", options);
 
+    // Type Annotations for TypeScript (when using Babylon as parser)
+    case "TSType":
+        throw new Error("unprintable type: " + JSON.stringify(n.type));
+
+    case "TSNumberKeyword":
+        return fromString("number", options);
+
+    case "TSObjectKeyword":
+        return fromString("object", options);
+
+    case "TSBooleanKeyword":
+        return fromString("boolean", options);
+
+    case "TSStringKeyword":
+        return fromString("string", options);
+
+    case "TSSymbolKeyword":
+        return fromString("symbol", options);
+
+    case "TSAnyKeyword":
+        return fromString("any", options);
+
+    case "TSVoidKeyword":
+        return fromString("void", options);
+
+    case "TSThisType":
+        return fromString("this", options);
+
+    case "TSNullKeyword":
+        return fromString("null", options);
+
+    case "TSUndefinedKeyword":
+        return fromString("undefined", options);
+
+    case "TSNeverKeyword":
+        return fromString("never", options);
+
+    case "TSArrayType":
+        return concat([
+            path.call(print, "elementType"),
+            "[]"
+        ]);
+
+    case "TSLiteralType":
+        return path.call(print, "literal")
+
+    case "TSUnionType":
+        return fromString(" | ").join(path.map(print, "types"));
+
+    case "TSIntersectionType":
+        return fromString(" & ").join(path.map(print, "types"));
+
+    case "TSConditionalType":
+        parts.push(
+            path.call(print, "checkType"),
+            " extends ",
+            path.call(print, "extendsType"),
+            " ? ",
+            path.call(print, "trueType"),
+            " : ",
+            path.call(print, "falseType")
+        );
+
+        return concat(parts);
+
+    case "TSInferType":
+        parts.push(
+            "infer ",
+            path.call(print, "typeParameter")
+        );
+
+        return concat(parts);
+
+    case "TSParenthesizedType":
+        return concat([
+            "(",
+            path.call(print, "typeAnnotation"),
+            ")"
+        ]);
+
+    case "TSFunctionType":
+    case "TSConstructorType":
+        return concat([
+            path.call(print, "typeParameters"),
+            "(",
+            printFunctionParams(path, options, print),
+            ")",
+            path.call(print, "typeAnnotation")
+        ]);
+
+    case "TSMappedType": {
+        parts.push(
+            n.readonly ? "readonly " : "",
+            "[",
+            path.call(print, "typeParameter"),
+            "]",
+            n.optional ? "?" : ""
+        );
+
+        if (n.typeAnnotation) {
+            parts.push(": ", path.call(print, "typeAnnotation"), ";");
+        }
+
+        return concat([
+            "{\n",
+            concat(parts).indent(options.tabWidth),
+            "\n}",
+        ]);
+    }
+
+    case "TSTupleType":
+        return concat([
+            "[",
+            fromString(", ").join(path.map(print, "elementTypes")),
+            "]"
+        ]);
+
+    case "TSIndexedAccessType":
+        return concat([
+            path.call(print, "objectType"),
+            "[",
+            path.call(print, "indexType"),
+            "]"
+        ]);
+
+    case "TSTypeOperator":
+        return concat([
+            path.call(print, "operator"),
+            " ",
+            path.call(print, "typeAnnotation")
+        ]);
+
+    case "TSTypeLiteral": {
+        const memberLines =
+            fromString(",\n").join(path.map(print, "members"));
+
+        if (memberLines.isEmpty()) {
+            return fromString("{}", options);
+        }
+
+        parts.push(
+            "{\n",
+            memberLines.indent(options.tabWidth),
+            "\n}"
+        );
+
+        return concat(parts);
+    }
+
+    case "TSEnumMember":
+        parts.push(path.call(print, "id"));
+        if (n.initializer) {
+            parts.push(
+                " = ",
+                path.call(print, "initializer")
+            );
+        }
+        return concat(parts);
+
+    case "TSTypeQuery":
+        return concat([
+            "typeof ",
+            path.call(print, "exprName"),
+        ]);
+
+    case "TSParameterProperty":
+        if (n.accessibility) {
+            parts.push(n.accessibility, " ");
+        }
+
+        if (n.export) {
+            parts.push("export ");
+        }
+
+        if (n.static) {
+            parts.push("static ");
+        }
+
+        if (n.readonly) {
+            parts.push("readonly ");
+        }
+
+        parts.push(path.call(print, "parameter"));
+
+        return concat(parts);
+
+    case "TSTypeReference":
+        return concat([
+            path.call(print, "typeName"),
+            path.call(print, "typeParameters")
+        ]);
+
+    case "TSQualifiedName":
+        return concat([
+            path.call(print, "left"),
+            ".",
+            path.call(print, "right")
+        ]);
+
+    case "TSAsExpression": {
+        var withParens = n.extra && n.extra.parenthesized === true;
+        parts = [];
+        if (withParens) parts.push("(");
+        parts.push(
+            path.call(print, "expression"),
+            fromString(" as "),
+            path.call(print, "typeAnnotation")
+        );
+        if (withParens) parts.push(")");
+
+        return concat(parts);
+    }
+
+    case "TSNonNullExpression":
+        return concat([
+            path.call(print, "expression"),
+            "!"
+        ]);
+
+    case "TSTypeAnnotation": {
+        // similar to flow's FunctionTypeAnnotation, this can be
+        // ambiguous: it can be prefixed by => or :
+        // in a type predicate, it takes the for u is U
+        var parent = path.getParentNode(0);
+        var prefix = ": ";
+        var isFunctionType = namedTypes.TSFunctionType.check(parent);
+        if (namedTypes.TSFunctionType.check(parent)) {
+            prefix = " => ";
+        }
+
+        if (namedTypes.TSTypePredicate.check(parent)) {
+            prefix = " is ";
+        }
+
+        return concat([
+            prefix,
+            path.call(print, "typeAnnotation")
+        ]);
+    }
+
+    case "TSIndexSignature":
+        return concat([
+            n.readonly ? "readonly " : "",
+            "[",
+            path.map(print, "parameters"),
+            "]",
+            path.call(print, "typeAnnotation")
+        ]);
+
+    case "TSPropertySignature":
+        parts.push(
+            printVariance(path, print),
+            n.readonly ? "readonly " : ""
+        );
+
+        if (n.computed) {
+            parts.push(
+                "[",
+                path.call(print, "key"),
+                "]"
+            );
+        } else {
+            parts.push(path.call(print, "key"));
+        }
+
+        parts.push(
+            n.optional ? "?" : "",
+            path.call(print, "typeAnnotation")
+        );
+
+        return concat(parts);
+
+    case "TSMethodSignature":
+        if (n.computed) {
+            parts.push(
+                "[",
+                path.call(print, "key"),
+                "]"
+            );
+        } else {
+            parts.push(path.call(print, "key"));
+        }
+
+        if (n.optional) {
+            parts.push("?");
+        }
+
+        parts.push(
+            path.call(print, "typeParameters"),
+            "(",
+            printFunctionParams(path, options, print),
+            ")",
+            path.call(print, "typeAnnotation")
+        );
+
+        return concat(parts);
+
+    case "TSTypePredicate":
+        return concat([
+            path.call(print, "parameterName"),
+            path.call(print, "typeAnnotation")
+        ]);
+
+    case "TSCallSignatureDeclaration":
+        return concat([
+            path.call(print, "typeParameters"),
+            "(",
+            printFunctionParams(path, options, print),
+            ")",
+            path.call(print, "typeAnnotation")
+        ]);
+
+    case "TSConstructSignatureDeclaration":
+        if (n.typeParameters) {
+            parts.push(
+                "new",
+                path.call(print, "typeParameters")
+            );
+        } else {
+            parts.push("new ");
+        }
+
+        parts.push(
+            "(",
+            printFunctionParams(path, options, print),
+            ")",
+            path.call(print, "typeAnnotation")
+        );
+
+        return concat(parts);
+
+    case "TSTypeAliasDeclaration":
+        return concat([
+            "type ",
+            path.call(print, "id"),
+            path.call(print, "typeParameters"),
+            " = ",
+            path.call(print, "typeAnnotation"),
+            ";"
+        ]);
+
+    case "TSTypeParameter":
+        parts.push(path.call(print, "name"));
+
+        // ambiguous because of TSMappedType
+        var parent = path.getParentNode(0);
+        var isInMappedType = namedTypes.TSMappedType.check(parent);
+
+        if (n.constraint) {
+            parts.push(
+                isInMappedType ? " in " : " extends ",
+                path.call(print, "constraint")
+            );
+        }
+
+        if (n["default"]) {
+            parts.push(" = ", path.call(print, "default"));
+        }
+
+        return concat(parts);
+
+    case "TSTypeAssertion":
+        var withParens = n.extra && n.extra.parenthesized === true;
+        if (withParens) {
+            parts.push("(");
+        }
+
+        parts.push(
+            "<",
+            path.call(print, "typeAnnotation"),
+            "> ",
+            path.call(print, "expression")
+        );
+
+        if (withParens) {
+            parts.push(")");
+        }
+
+        return concat(parts);
+
+    case "TSTypeParameterDeclaration":
+    case "TSTypeParameterInstantiation":
+        return concat([
+            "<",
+            fromString(", ").join(path.map(print, "params")),
+            ">"
+        ]);
+
+    case "TSEnumDeclaration":
+        parts.push(
+            n.declare ? "declare " : "",
+            n.const ? "const " : "",
+            "enum ",
+            path.call(print, "id")
+        );
+
+        const memberLines =
+            fromString(",\n").join(path.map(print, "members"));
+
+        if (memberLines.isEmpty()) {
+            parts.push(" {}");
+        } else {
+            parts.push(
+                " {\n",
+                memberLines.indent(options.tabWidth),
+                "\n}"
+            );
+        }
+
+        return concat(parts);
+
+    case "TSExpressionWithTypeArguments":
+        return concat([
+            path.call(print, "expression"),
+            path.call(print, "typeParameters")
+        ]);
+
+    case "TSInterfaceBody":
+        var lines = fromString(";\n").join(path.map(print, "body"));
+        if (lines.isEmpty()) {
+            return fromString("{}", options);
+        }
+
+        return concat([
+            "{\n",
+            lines.indent(options.tabWidth), ";",
+            "\n}",
+        ]);
+
+    case "TSImportEqualsDeclaration":
+        if (n.isExport) {
+            parts.push("export ");
+        }
+
+        parts.push(
+            "import ",
+            path.call(print, "id"),
+            " = ",
+            path.call(print, "moduleReference")
+        );
+
+        return maybeAddSemicolon(concat(parts));
+
+    case "TSExternalModuleReference":
+      return concat(["require(", path.call(print, "expression"), ")"]);
+
+    case "TSModuleDeclaration": {
+        const parent = path.getParentNode();
+
+        if (parent.type === "TSModuleDeclaration") {
+            parts.push(".");
+        } else {
+            if (n.declare) {
+                parts.push("declare ");
+            }
+
+            if (! n.global) {
+                const isExternal = n.id.type === "StringLiteral" ||
+                    (n.id.type === "Literal" &&
+                     typeof n.id.value === "string");
+
+                if (isExternal) {
+                    parts.push("module ");
+
+                } else if (n.loc &&
+                           n.loc.lines &&
+                           n.id.loc) {
+                    const prefix = n.loc.lines.sliceString(
+                        n.loc.start,
+                        n.id.loc.start
+                    );
+
+                    // These keywords are fundamentally ambiguous in the
+                    // Babylon parser, and not reflected in the AST, so
+                    // the best we can do is to match the original code,
+                    // when possible.
+                    if (prefix.indexOf("module") >= 0) {
+                        parts.push("module ");
+                    } else {
+                        parts.push("namespace ");
+                    }
+
+                } else {
+                    parts.push("namespace ");
+                }
+            }
+        }
+
+        parts.push(path.call(print, "id"));
+
+        if (n.body && n.body.type === "TSModuleDeclaration") {
+            parts.push(path.call(print, "body"));
+        } else if (n.body) {
+            const bodyLines = path.call(print, "body");
+            if (bodyLines.isEmpty()) {
+                parts.push(" {}");
+            } else {
+                parts.push(
+                    " {\n",
+                    bodyLines.indent(options.tabWidth),
+                    "\n}"
+                );
+            }
+        }
+
+        return concat(parts);
+    }
+
+    case "TSModuleBlock":
+        return path.call(function (bodyPath) {
+            return printStatementSequence(bodyPath, options, print);
+        }, "body");
+
     // Unhandled types below. If encountered, nodes of these types should
     // be either left alone or desugared into AST types that are fully
     // supported by the pretty-printer.
@@ -1769,22 +2422,37 @@ function printMethod(path, options, print) {
     var kind = node.kind;
     var parts = [];
 
-    if (node.type === "ObjectMethod" || node.type === "ClassMethod") {
-        node.value = node;
-    } else {
-        namedTypes.FunctionExpression.assert(node.value);
+    var nodeValue = node.value;
+    if (! namedTypes.FunctionExpression.check(nodeValue)) {
+        nodeValue = node;
     }
 
-    if (node.value.async) {
+    var access = node.accessibility || node.access;
+    if (typeof access === "string") {
+        parts.push(access, " ");
+    }
+
+    if (node.static) {
+        parts.push("static ");
+    }
+
+    if (node.abstract) {
+        parts.push("abstract ");
+    }
+
+    if (node.readonly) {
+        parts.push("readonly ");
+    }
+
+    if (nodeValue.async) {
         parts.push("async ");
     }
 
-    if (!kind || kind === "init" || kind === "method" || kind === "constructor") {
-        if (node.value.generator) {
-            parts.push("*");
-        }
-    } else {
-        assert.ok(kind === "get" || kind === "set");
+    if (nodeValue.generator) {
+        parts.push("*");
+    }
+
+    if (kind === "get" || kind === "set") {
         parts.push(kind, " ");
     }
 
@@ -1793,18 +2461,44 @@ function printMethod(path, options, print) {
         key = concat(["[", key, "]"]);
     }
 
-    parts.push(
-        key,
-        path.call(print, "value", "typeParameters"),
-        "(",
-        path.call(function(valuePath) {
-            return printFunctionParams(valuePath, options, print);
-        }, "value"),
-        ")",
-        path.call(print, "value", "returnType"),
-        " ",
-        path.call(print, "value", "body")
-    );
+    parts.push(key);
+
+    if (node.optional) {
+        parts.push("?");
+    }
+
+    if (node === nodeValue) {
+        parts.push(
+            path.call(print, "typeParameters"),
+            "(",
+            printFunctionParams(path, options, print),
+            ")",
+            path.call(print, "returnType")
+        );
+
+        if (node.body) {
+            parts.push(" ", path.call(print, "body"));
+        } else {
+            parts.push(";");
+        }
+
+    } else {
+        parts.push(
+            path.call(print, "value", "typeParameters"),
+            "(",
+            path.call(function(valuePath) {
+                return printFunctionParams(valuePath, options, print);
+            }, "value"),
+            ")",
+            path.call(print, "value", "returnType")
+        );
+
+        if (nodeValue.body) {
+            parts.push(" ", path.call(print, "value", "body"));
+        } else {
+            parts.push(";");
+        }
+    }
 
     return concat(parts);
 }
@@ -1829,9 +2523,13 @@ function printArgumentsList(path, options, print) {
 function printFunctionParams(path, options, print) {
     var fun = path.getValue();
 
-    namedTypes.Function.assert(fun);
-
-    var printed = path.map(print, "params");
+    if (fun.params) {
+        var params = fun.params;
+        var printed = path.map(print, "params");
+    } else if (fun.parameters) {
+        params = fun.parameters;
+        printed = path.map(print, "parameters");
+    }
 
     if (fun.defaults) {
         path.each(function(defExprPath) {
@@ -1853,7 +2551,7 @@ function printFunctionParams(path, options, print) {
         joined = fromString(",\n").join(printed);
         if (util.isTrailingCommaEnabled(options, "parameters") &&
             !fun.rest &&
-            fun.params[fun.params.length - 1].type !== 'RestElement') {
+            params[params.length - 1].type !== 'RestElement') {
             joined = concat([joined, ",\n"]);
         } else {
             joined = concat([joined, "\n"]);
@@ -1864,42 +2562,12 @@ function printFunctionParams(path, options, print) {
     return joined;
 }
 
-function printObjectMethod(path, options, print) {
-    var objMethod = path.getValue();
-    var parts = [];
-
-    if (objMethod.async)
-        parts.push("async ");
-
-    if (objMethod.generator)
-        parts.push("*");
-
-    if (objMethod.method || objMethod.kind === "get" || objMethod.kind === "set") {
-        return printMethod(path, options, print);
-    }
-
-    var key = path.call(print, "key");
-    if (objMethod.computed) {
-        parts.push("[", key, "]");
-    } else {
-        parts.push(key);
-    }
-
-    parts.push(
-        "(",
-        printFunctionParams(path, options, print),
-        ")",
-        path.call(print, "returnType"),
-        " ",
-        path.call(print, "body")
-    );
-
-    return concat(parts);
-}
-
 function printExportDeclaration(path, options, print) {
     var decl = path.getValue();
     var parts = ["export "];
+    if (decl.exportKind && decl.exportKind !== "value") {
+        parts.push(decl.exportKind + " ");
+    }
     var shouldPrintSpaces = options.objectCurlySpacing;
 
     namedTypes.Declaration.assert(decl);
@@ -1932,14 +2600,15 @@ function printExportDeclaration(path, options, print) {
     }
 
     var lines = concat(parts);
-
     if (lastNonSpaceCharacter(lines) !== ";" &&
         ! (decl.declaration &&
            (decl.declaration.type === "FunctionDeclaration" ||
-            decl.declaration.type === "ClassDeclaration"))) {
+            decl.declaration.type === "ClassDeclaration" ||
+            decl.declaration.type === "TSModuleDeclaration" ||
+            decl.declaration.type === "TSInterfaceDeclaration" ||
+            decl.declaration.type === "TSEnumDeclaration"))) {
         lines = concat([lines, ";"]);
     }
-
     return lines;
 }
 
@@ -1961,6 +2630,26 @@ function printFlowDeclaration(path, parts) {
     return concat(parts);
 }
 
+function printVariance(path, print) {
+    return path.call(function (variancePath) {
+        var value = variancePath.getValue();
+
+        if (value) {
+            if (value === "plus") {
+                return fromString("+");
+            }
+
+            if (value === "minus") {
+                return fromString("-");
+            }
+
+            return print(variancePath);
+        }
+
+        return fromString("");
+    }, "variance");
+}
+
 function adjustClause(clause, options) {
     if (clause.length > 1)
         return concat([" ", clause]);
