@@ -1666,16 +1666,11 @@ describe("printer", function() {
       check("let i = 0 ; ; ", parser);
     }
 
-    checkWith(require("esprima"));
-
-    try {
-      checkWith(require("reify/lib/parsers/acorn.js"));
-      checkWith(require("reify/lib/parsers/babylon.js"));
-    } catch (e) {
-      if (require("semver").gte(process.version, "4.0.0")) {
-        throw e;
-      }
-    }
+    checkWith(require("../parsers/esprima"));
+    checkWith(require("../parsers/acorn"));
+    checkWith(require("../parsers/babylon"));
+    checkWith(require("../parsers/typescript"));
+    checkWith(require("../parsers/flow"));
   });
 
   it("parenthesizes NumericLiteral MemberExpression objects", function () {
@@ -1699,4 +1694,31 @@ describe("printer", function() {
       "(1).foo"
     );
   });
+
+  it("prints numbers in bases other than 10 without converting them", function() {
+    var code = [
+      'let decimal = 6;',
+      'let hex = 0xf00d;',
+      'let binary = 0b1010;',
+      'let octal = 0o744;'
+    ].join(eol);
+    var ast = parse(code);
+    var printer = new Printer({});
+    var pretty = printer.printGenerically(ast).code;
+
+    assert.strictEqual(pretty, code);
+  });
+
+  it("reprints modified numeric literals", function () {
+    const code = '3 + 4;';
+    const ast = parse(code);
+    const expr = ast.program.body[0].expression;
+    const left = expr.left;
+    const right = expr.right;
+
+    left.value++;
+    right.value++;
+
+    assert.strictEqual(recast.print(ast).code, '4 + 5;');
+  });
 });
